---
title: Channel Definition Format
---

The Channel Definition Format was a Microsoft file format for syndication
based on XML.  It was designed for its *Active Channels* technology and got
extended for *Active Desktop* integration or for [OSD][osd].

## Specification (or lack thereof)

CDF does not have a standardized specification: [a note][w3c] was sent to the
W3C in 1997 with a very early draft of the format, with the intention of later
making it into a standard, but there were no further updates.
The [Microsoft docs][docs-970314] of that time are the same as this draft.

The [latest documentation][docs], from 2002, describes a format that varies a
lot from that W3C note.

There is no official <abbr title="Document Type Definition">DTD</abbr> or
<abbr title="XML Schema Definition">XSD</abbr>.  Someone wrote a [DTD][dtd]
with support for the *Active Desktop* and [OSD][osd] extensions.

I wrote my own XSD files to help me use this format in [ITSB][itsb]:

* [W3C draft](./xsd/cdf-w3c.xsd)
* [CDF 0.4](./xsd/cdf.xsd) (latest)

The CDF 0.4 XSD includes the
[CDF Extensions for Active Desktop Items][active-desktop-ext].

## Support

CDF was supported by Internet Explorer 4 to 7, and no other software is known
to have supported this format.

The format was created and implemented in IE4 while XML was still a draft,
causing some valid XML documents to not be understood properly by IE4, and all
the sample CDF files provided with IE4 to make XML parsers scream.

Support was dropped in 2009 with the release of IE8 as it lost against RSS,
and as Microsoft's dedicated RSS team implemented RSS support in IE7.

## Notable features

* Scheduling options to avoid useless synchronizations
* Nested channels (merge multiple feeds into one)
* Custom icons in the IE favorites sidebar
* Customization of IE's crawling behavior when making a website available
  offline, using `<LINK REL="OFFLINE" HREF="...cdf">`
* *Active Desktop* integration to display articles on the desktop
* Integration with [Open Software Description][osd] to send software updates

## Usage

I use this format in [ITSB][itsb] to provide separate channels generated from
each RSS feed, or channels that group channels together.  This second form
usually causes Internet Explorer to lag heavily due to the few megabytes of
items it is downloading in a single file.  All of those CDF files have been
tested and confirmed to be working with Internet Explorer 6 SP1 on Windows
2000 SP4.

## Links

* [March 10th 1997 specification][docs-970314]
* [March 21st 1997 specification][docs-970321]
* [Latest documentation][docs] (CDF 0.4)
* [Extensions for Active Desktop items][active-desktop-ext]
* [Unofficial DTD][dtd]
* [W3C draft][w3c]
* Special Edition Using HTML 4, [Building Channels with CDF][building],
  Jerry Honeycutt

[active-desktop-ext]: https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/platform-apis/aa768141%28v=vs.85%29
[building]: https://www.rigacci.org/docs/biblio/online/html4/ch30/ch30.htm
[docs]: https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/platform-apis/aa768139(v=vs.85)
[docs-970314]: http://xml.coverpages.org/cdf970314.html
[docs-970321]: http://xml.coverpages.org/cdf970321.html
[dtd]: http://www2.cmp.uea.ac.uk/~jrwg/JAXP/examples/cdf/cdf.dtd
[itsb]: https://tilde.town/~lucidiot/itsb/
[osd]: ./osd.html
[w3c]: https://www.w3.org/TR/NOTE-CDFsubmit.html
